/* this program is to simulate the real world attacks and generates tcp, udp and * icmp packets test trigger for ourmon. 
 *
 * To test the triggers, first run test tool, then start ourmon. Or run ourmon
 * firstly and then start test tool. After test tool run 30 seconds, check
 * if mon.lite or dump files have been created, if yes, either wait for
 * another more 30 seconds or immediately stop test tool to test, so on and so 
 * forth. If there are no dump files or the length of files is weird, there must * be some failures.
 *
 * syntax:
 *	tri [-tS][-tF][-fI][-fT][-fU][-sI][-sP][-sT][-sU][-u] host [hostname]
	    port [port]
 *	
 *	-tS : For tworm attrack. It generates tcp syn packets, the src addr of
 *	      of the packet is randomly generated. The dest addr is [hostname].
 *      -tF:  For tworm attack. It generates tcp fin packets. the src addr of
 *		of the packets are randomly generated. The dest addrs are 
 *		[hostname].
 *	-fI: For flow count trigger. It will firstly generate 10000 tcp packets  *	     then udp, then icmp packets. The src addrs and dest addrs for these *	     packets are the same.
 *      -fT: For flow count trigger. It only generates 10000 tcp packets. All
 *	     these packets have same src addr and dest addr.
 *	-fU: For flow count trigger. It only generate 10000 udp packets. All
 *	     these packets have same src addr and dest addr.
 *	-sI: For ip_scan trigger. It will generate icmp packets. These packets
 *	     have same src address but randomly generated different dest addr.
 *	-sT: For tcp_port_scan trigger. It will generate tcp packets. These
 *	     packets have same src address but randomly generately different 
 *	     dest port.
 *	-sU: For udp_port_scan trigger. It will generate udp packets. These 
 *	     packets have same src address but randomly generately different
 *	     udp dest port.
 *	-u:  For udp error trigger. It will generate icmp redirect packets and 
 *  	     udp packets that appear to come back to the src. These 
 *	     packets have different dest port. (TBD) 
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/errno.h>
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <netinet/tcp.h>
#include <netinet/udp.h>
#include <netinet/ip_icmp.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#define COUNT 40		/* tworm count */

#define TWORM_SYN 0x1
#define TWORM_FIN 0x2

#define FLOW_TCP 0x1
#define FLOW_UDP 0x2
#define FLOW_IP 0x3

#define SCAN_IP 0x1
#define SCAN_PORT 0x2
#define SCAN_TPORT 0x3
#define SCAN_UPORT 0x4

int tworm_flag = 0;
int flow_flag = 0;
int udperror_flag = 0;
int scan_flag = 0;

enum icmp_type {ECHO, REPLY, UNREACHABLE, REDIRECT};
enum packet_type {TCP_PACK, UDP_PACK, ICMP_PACK};
enum tcp_packet_type {TWORM_PACKET, FLOW_TCP_PACKET,SCAN_TPORT_PACKET};
enum udp_packet_type {FLOW_UDP_PACKET, SCAN_UDP_PACKET,SCAN_UPORT_PACKET, 
			UDP_ERROR_PACKET};
enum icmp_packet_type {FLOW_ICMP_PACKET, SCAN_ICMP_PACKET};

/* tcp pseudo header */
struct tcp_pseudohdr
{
        struct in_addr saddr;   /* src address */
        struct in_addr daddr;   /* dest address */
        u_char zero;
        u_char protocol;        /* protocol */
        u_short length;         /* length of packet */
        struct tcphdr tcpheader;
};      /* tcp header */

/* udp pseudo header */
struct udp_pseudohdr
{
        struct in_addr saddr;   /* src address */
        struct in_addr daddr;   /* dest address */
        u_char zero;
        u_char protocol;        /* protocol */
        u_short length;         /* length of packet */
        struct udphdr udpheader;
};      /* udp header */

/* udp pseudo header */
struct icmp_pseudohdr
{
        struct in_addr saddr;   /* src address */
        struct in_addr daddr;   /* dest address */
        u_char zero;
        u_char protocol;        /* protocol */
        u_short length;         /* length of packet */
        struct icmp icmpheader;
};      /* icmp header */

struct packet_buf {
	char buffer[40];
	int pkt_len;
};

void print_usage()
{
	fprintf(stderr,
		 "testri [-sT][-sF][-fI][-fT][-fU][-h] host [hostname] port [port#]\n\
  -tS host [hostname]\tgenerate tcp syn packets. The dest addr is [hostname]\n\
  -tF host [hostname]\tgenerate tcp fin packets. The dest addr is [hostname]\n\
  -fI\tFor flow count trigger. It will generate tcp,udp and icmp pacekts.\n\
  -fT\tFor flow count trigger. It will generate tcp packets. And all these packets have same src and dest address\n\
  -fU\tFor flow count trigger. It will generate udp packets. And all these packets have same src and dest address.\n\
  -sI\tFor ip_scan trigger. It will generate icmp packets. These packets have same src address.\n\
  -sT\tFor tcp_port_scan. It will generate tcp packets with same src address but random different port number\n\
  -sU\tfor udp_port_scan trigger. It will generate udp packets with same src address but random different port number.\n\
  -u\tfor udp error trigger\n\
  -h\thelp\n");
}

/* 
 * parse the command, get the parameters 
 * ip, dport : parameter which might be needed by attack methods
 * host, port: where to send packets
 */
void parse_para(char **argv, int argc, char *host, int *port)
{
	int i=0;

	for(i=1 ; i < argc; i++) {
		if(strcmp(argv[i], "-tS") == 0) {
			tworm_flag |= TWORM_SYN;
		}
		else if(strcmp(argv[i],"-tF") == 0) {
			tworm_flag |= TWORM_FIN; 
		}
		else if(strcmp(argv[i], "-fT") == 0) {
			flow_flag |= FLOW_TCP;	
		}
		else if(strcmp(argv[i], "-fU") == 0) {
			flow_flag |= FLOW_UDP;
		}
		else if(strcmp(argv[i], "-fI") == 0) {
			flow_flag |= FLOW_IP;
		}
		else if(strcmp(argv[i], "-sI") == 0) {
			scan_flag |= SCAN_IP; 
		}
		else if(strcmp(argv[i], "-sP") == 0) {
			scan_flag |= SCAN_PORT;
		}
		else if(strcmp(argv[i], "-sT") == 0) {
			scan_flag |= SCAN_TPORT;
		}
		else if(strcmp(argv[i], "-sU") == 0) {
			scan_flag |= SCAN_UPORT;
		}
		else if(strcmp(argv[i], "-u") == 0) {
			udperror_flag ++;
		}
		else if(strcmp(argv[i], "host") == 0) {
			strcpy(host, argv[++i]);
		}
		else if(strcmp(argv[i], "port") == 0) {
			*port = atoi(argv[++i]);
		}
		else {
			print_usage();
		}
	}	
}

/* caculate checksum for packet */
u_short checksum(u_short * data,u_short length)
{
        register long value;
        u_short i;

        for(i=0;i<(length>>1);i++)
                value+=data[i];

        if((length&1)==1)
                value+=(data[i]<<8);

        value=(value&65535)+(value>>16);

        return(~value);
}

/* generate tcp packet 
 * input: src, dest
 * output: packet
 */
void tcp_packet(struct sockaddr_in *src, struct sockaddr_in *dest, 
	struct packet_buf *packet)
{
	int len;
	struct ip * ipheader = (struct ip*)packet->buffer;
	struct tcphdr * tcpheader =(struct tcphdr*)(packet->buffer+sizeof(struct ip));
	struct tcp_pseudohdr pseudoheader;

	bzero(packet->buffer, sizeof(struct ip) + sizeof(struct tcphdr));
        ipheader->ip_v=IPVERSION;
        ipheader->ip_hl=sizeof(struct ip)/4;
        ipheader->ip_len=sizeof(struct ip)+sizeof(struct tcphdr);
        ipheader->ip_id=htons(0xF1C);
        ipheader->ip_ttl=255;
        ipheader->ip_p=IPPROTO_TCP;
        ipheader->ip_src = src->sin_addr;
        ipheader->ip_dst = dest->sin_addr;
#ifdef LINUX
#define th_sport source
#define th_dport dest
#define th_seq seq
#define th_off doff
#define th_win window
#define th_sum check
#endif
        tcpheader->th_sport= src->sin_port;
        tcpheader->th_dport= src->sin_port;
        tcpheader->th_seq=htonl(0xF1C);
	if(tworm_flag & TWORM_FIN) {
#ifdef LINUX
		tcpheader->fin = 1;
#else
		tcpheader->th_flags = TH_FIN;
#endif
	} 
	else { 
#ifdef LINUX
		tcpheader->syn = 1;
#else
        	tcpheader->th_flags=TH_SYN;
#endif
	}
        tcpheader->th_off=sizeof(struct tcphdr)/4;
        tcpheader->th_win=htons(2048);

        bzero(&pseudoheader,12+sizeof(struct tcphdr));
        pseudoheader.saddr.s_addr= src->sin_addr.s_addr;
        pseudoheader.daddr.s_addr= dest->sin_addr.s_addr;
        pseudoheader.protocol=IPPROTO_TCP;
	pseudoheader.length=htons(sizeof(struct tcphdr));
        bcopy((char *) tcpheader,(char *) &pseudoheader.tcpheader,sizeof(struct
tcphdr));
    	tcpheader->th_sum=checksum((u_short *) &pseudoheader,
			12+sizeof(struct tcphdr));

	packet->pkt_len = sizeof(struct ip) + sizeof(struct tcphdr);
}

int conn(char *host, int port, int *sock, struct sockaddr_in *sin, 
		enum packet_type type)
{
	struct hostent *hoste;
	int hincl = 1;
	
        if((hoste=gethostbyname(host))!=NULL)
                bcopy(hoste->h_addr,&sin->sin_addr,hoste->h_length);
        else if((sin->sin_addr.s_addr=inet_addr(host))==-1)
        {
                fprintf(stderr,"unknown host %s\n",host);
		exit(-1);
        }

	if((sin->sin_port=htons(port))==0)
        {
                fprintf(stderr,"unknown port %d\n",port);
		exit(-1);
        }

	switch(type) {
		case TCP_PACK:	/* TCP */ 
   		  if((*sock=socket(AF_INET,SOCK_RAW,IPPROTO_TCP))==-1)
        	  {
                	fprintf(stderr,"couldn't allocate raw socket\n");
			exit(-1);
        	  }
		  break;
		case UDP_PACK:
		  if((*sock=socket(AF_INET,SOCK_RAW,IPPROTO_UDP))==-1)
        	  {
                	fprintf(stderr,"couldn't allocate raw socket\n");
			exit(-1);
        	  }
		  break;
		case ICMP_PACK:
        	  if((*sock=socket(AF_INET,SOCK_RAW,IPPROTO_ICMP))==-1)
        	  {
                	fprintf(stderr,"couldn't allocate raw socket\n");
                	return;
        	  }
		  break;
	}
	return 0;
}

/* generate and send tons of packets 
 * flag : different packets for different attacks
 *	1 - tworm packet
 *	2 - flow packet
 */
void tcp_horse(char *host, int port, enum tcp_packet_type flag) 
{
	struct packet_buf pkt;		/* packet */
	struct sockaddr_in sin_src;	/* src address */
	struct sockaddr_in sin_dest;	/* dest address */
	struct sockaddr_in sin_ddest;
	int i, j, arr[COUNT],cond;
	int hincl = 1;
	int sock;

	bzero(&sin_src, sizeof(struct sockaddr_in));
	sin_src.sin_family = AF_INET;

	bzero(&sin_dest, sizeof(struct sockaddr_in));
	sin_dest.sin_family = AF_INET;

	/* set the socket, and get dest address*/
	conn(host, port, &sock, &sin_dest, 1);
	sin_ddest = sin_dest;

	/* src address is random */
	for(i = 0; i < COUNT; i++) {
		arr[i] = rand();
	}

	/* send packets */	
	j = 0;
	
	while(1 ) {
	
		if(flag==TWORM_PACKET || 
			(flag==FLOW_TCP_PACKET && flow_flag&FLOW_IP==0)) {
				;
		}
		else {
			/* only send 100000 packets */
			if(j == 100000)
				break; 
		} 
				
		/* For tworm, src address is random 
		 * for flow, src address is fixed( TBD) 
		 */
		switch (flag) {
			case TWORM_PACKET:	/* tworm */
				sin_src.sin_addr.s_addr = htonl(arr[(j++)%COUNT]);
				sin_src.sin_port = htons(port);
				break;
			case FLOW_TCP_PACKET: /* flow */
				sin_src.sin_addr.s_addr = htonl(arr[(j++)/1000]);
				sin_src.sin_port = htons(port);
				break;									case SCAN_TPORT_PACKET: /* scan */
				sin_dest.sin_addr.s_addr = htonl(2000*arr[(j++)%COUNT]);
				sin_dest.sin_port=htons(arr[(j++)%COUNT]);
				sin_src.sin_addr.s_addr=htonl(2000*arr[1]);
				sin_src.sin_port=htons(arr[(j++)%COUNT]);
				break;
		}
		bzero(&pkt, sizeof(struct packet_buf));
		/* construct a tcp packet */
		tcp_packet(&sin_src, &sin_dest, &pkt);
	        if (setsockopt(sock, IPPROTO_IP, IP_HDRINCL, &hincl, sizeof(hincl)))
        	{
          		fprintf(stderr,"Unable to set IP_HDRINCL to 1: %s",strerror(errno));
          		exit(-1);
        	}
		/* send the packet */
		if(sendto(sock, pkt.buffer, pkt.pkt_len, 0, 
		    (struct sockaddr *)&sin_ddest, sizeof(struct sockaddr_in))
				< 0 ) {
			fprintf(stderr,"could not send packet:%s\n", 
					strerror(errno));
			exit(-1);
		} 		
	}
}

void udp_packet(struct sockaddr_in *src,struct sockaddr_in *dest, 
		struct packet_buf *packet)
{
        struct ip * ipheader=(struct ip *) packet->buffer;
        struct udphdr * udpheader=(struct udphdr *) (packet->buffer+sizeof(struct ip));
        struct udp_pseudohdr pseudoheader;

        bzero(&packet->buffer,sizeof(struct ip)+sizeof(struct tcphdr));

        ipheader->ip_v=IPVERSION;
        ipheader->ip_hl=sizeof(struct ip)/4;
        ipheader->ip_len=sizeof(struct ip)+sizeof(struct udphdr);
        ipheader->ip_id=htons(0xF1C);
        ipheader->ip_ttl=255;
        ipheader->ip_p=IPPROTO_UDP;
        ipheader->ip_src= src->sin_addr;
        ipheader->ip_dst= dest->sin_addr;

#ifdef LINUX
#define uh_sport source
#define uh_dport dest
#define uh_ulen   len
#define uh_sum   check
#endif
        udpheader->uh_sport = src->sin_port;
        udpheader->uh_dport = dest->sin_port;
        udpheader->uh_ulen = htons((u_short)(sizeof(struct udphdr) + 8));

        bzero(&pseudoheader,12+sizeof(struct udphdr));
        pseudoheader.saddr.s_addr= src->sin_addr.s_addr;
        pseudoheader.daddr.s_addr= dest->sin_addr.s_addr;
        pseudoheader.protocol=IPPROTO_UDP;
        pseudoheader.length=htons(sizeof(struct udphdr));
        bcopy((char *) udpheader,(char *) &pseudoheader.udpheader,sizeof(struct
udphdr));
        udpheader->uh_sum=checksum((u_short *) &pseudoheader,
		12+sizeof(struct udphdr));

	packet->pkt_len = sizeof(struct ip) + sizeof(struct udphdr) + 12;
}

/* return the lengtho of icmp packet 
 * according its type
 */
int icmp_data_len(enum icmp_type type)
{
        int len = 0;

        switch (type) {
                case ECHO:
                        len = 56;
                        break;
                case REPLY:
                        len = 56; /*TBD*/
                        break;
		case UNREACHABLE:
			len = 56; /*TBD*/
			break;
		case REDIRECT:
			len = 8;
			break;
                default:
                        len = 0;
        }

        return len;
}


int icmp_data_type(enum icmp_type type)
{
        int ret = 0;

        switch (type) {
                case ECHO:
                        ret = ICMP_ECHO;
                        break;
                case REPLY:
                        ret = ICMP_ECHOREPLY;
                        break;
                case UNREACHABLE:
                        ret = ICMP_UNREACH;
                        break;
		case REDIRECT:
			ret = ICMP_REDIRECT;
			break;
        }

        return ret;
}


void icmp_packet(struct sockaddr_in *src, struct sockaddr_in *dest, 
	struct packet_buf *packet, enum icmp_type type)
{
        struct ip * ipheader=(struct ip *) packet->buffer;
        struct icmp * icmpheader=(struct icmp *) (packet->buffer+sizeof(struct ip));
        struct icmp_pseudohdr pseudoheader;
        pid_t pid;
	static int j=0;
	int datalen;
	int i;

	datalen = icmp_data_len(type);
        pid = getpid();
        bzero(&packet->buffer,sizeof(struct ip)+sizeof(struct icmp));
	ipheader->ip_v=IPVERSION;
        ipheader->ip_hl=sizeof(struct ip)/4;
        ipheader->ip_len=sizeof(struct ip)+8+datalen;
        ipheader->ip_id=htons(0xF1C);
        ipheader->ip_ttl=255;
        ipheader->ip_p=IPPROTO_ICMP;
        ipheader->ip_src = src->sin_addr;
        ipheader->ip_dst = dest->sin_addr;

        icmpheader->icmp_type = icmp_data_type(type);

	switch(type) {
		case ECHO:
 		case REPLY:
		case UNREACHABLE:
        		icmpheader->icmp_code = 0;
        		icmpheader->icmp_id = pid;
        		icmpheader->icmp_seq = j++;
        		gettimeofday((struct timeval *)icmpheader->icmp_data, NULL);
			break;
		case REDIRECT:
			icmpheader->icmp_code = 3;
			icmpheader->icmp_gwaddr = dest->sin_addr;
			break;
			
	}

        bzero(&pseudoheader,12+sizeof(struct icmp));
        pseudoheader.saddr.s_addr = src->sin_addr.s_addr;
        pseudoheader.daddr.s_addr = dest->sin_addr.s_addr;
        pseudoheader.protocol = IPPROTO_ICMP;
        pseudoheader.length = htons(8+datalen);
        bcopy((char *) icmpheader,(char *) &pseudoheader.icmpheader,
			sizeof(struct icmp));
        icmpheader->icmp_cksum=checksum((u_short *) &pseudoheader,8+datalen);

	packet->pkt_len = sizeof(struct ip) + 8 + datalen;

} 
 
void udp_horse(char *host, int port,enum udp_packet_type flag)
{
	struct packet_buf pkt;          /* packet */
        struct sockaddr_in sin_src;     /* src address */
        struct sockaddr_in sin_dest;    /* dest address */
	struct sockaddr_in sin_ddest;
        int i, j, arr[COUNT],portno[COUNT],cond;
        int sock;
	int hincl = 1;

	/* src address is random */
	for(i = 0; i < COUNT; i++) {
		arr[i] = rand();
		portno[i] = 250+i;
	}

        bzero(&sin_src, sizeof(struct sockaddr_in));
        sin_src.sin_family = AF_INET;

        bzero(&sin_dest, sizeof(struct sockaddr_in));
        sin_dest.sin_family = AF_INET;

	bzero(&sin_ddest, sizeof(struct sockaddr_in));
	sin_ddest.sin_family = AF_INET;

        /* set the socket, and get dest address*/
        conn(host, port, &sock, &sin_dest, UDP_PACK);
	sin_ddest = sin_dest;
	
	/* send packets */
	/* the number of packets
	 * if tool is only used for testing topn_ip(udp)
	 * the number of packets could be more
	 */
        j = 0;

        while( 1 ) {

		if((flag==FLOW_UDP_PACKET && (flow_flag&FLOW_IP) == 0)) 
			;
		else  {
			if( j == 100000 ) 
				break;
		}

                /* For tworm, src address is random
                 * for flow, src address is fixed( TBD)
                 */
                switch (flag) {
                        case FLOW_UDP_PACKET: /* flow */
                                sin_src.sin_addr.s_addr = htonl(arr[(j++)/1000]);
                                sin_src.sin_port = htons(port);
                                break;
			case SCAN_UDP_PACKET:
				/* one src, many ip address */
				sin_src.sin_addr.s_addr = htonl(arr[1]);
				sin_src.sin_port = htons(arr[(j++)%COUNT]);
				sin_dest.sin_addr.s_addr=htonl(arr[(j++)%COUNT]);
				sin_dest.sin_port = htons(arr[1]);
				break;
			case SCAN_UPORT_PACKET:
				//sin_src.sin_addr.s_addr = htonl(arr[(j++)%COUNT]);
				sin_src = sin_ddest;
				sin_src.sin_port=htons(200);
				sin_dest.sin_addr.s_addr=htonl(arr[1]);
                                sin_dest.sin_port = htons(portno[(j++)%COUNT]);
				break;
			case UDP_ERROR_PACKET:
				sin_src = sin_dest;
				sin_src.sin_port = htons(portno[2]);
                                sin_dest.sin_port = htons(portno[(j++)%COUNT]);
				break;
                }
                /* construct a tcp packet */
                udp_packet(&sin_src, &sin_dest, &pkt);
	        if (setsockopt(sock, IPPROTO_IP, IP_HDRINCL, &hincl, sizeof(hincl)))
        	{
          		fprintf(stderr,"Unable to set IP_HDRINCL to 1: %s",strerror(errno));
          		exit(-1);
        	}
		if(sendto(sock, pkt.buffer, pkt.pkt_len, 0, 
		  (struct sockaddr *) &sin_ddest, sizeof(struct sockaddr_in))<0)
		{
			fprintf(stderr,"couldn't send packet:%s\n", 
				strerror(errno));
			exit(-1);
		}	
	 }
}

void icmp_horse(char *host, int port, enum icmp_type type, 
	enum icmp_packet_type flag)
{

	struct packet_buf pkt;          /* packet */
        struct sockaddr_in sin_src;     /* src address */
        struct sockaddr_in sin_dest;    /* dest address */
	struct sockaddr_in sin_ddest;
        int i, j;
        int sock;
	int hincl = 1;
	unsigned int addr[COUNT],portno[COUNT];

	for(i = 0 ; i < COUNT ; i++) {
		addr[i] = rand();
		portno[i] = 250 + i;
	}

        bzero(&sin_src, sizeof(struct sockaddr_in));
        sin_src.sin_family = AF_INET;

        bzero(&sin_dest, sizeof(struct sockaddr_in));
        sin_dest.sin_family = AF_INET;

        /* set the socket, and get dest address*/
        conn(host, port, &sock, &sin_dest, ICMP_PACK);
	sin_ddest = sin_dest;

        j = 0;
        while( 1 )
	 {
		if(flag == FLOW_ICMP_PACKET && (flow_flag & FLOW_IP) == 0) 
			;
		else {
			if( j == 100000 ) 
				break;
		}

                /* For tworm, src address is random
                 * for flow, src address is fixed( TBD)
                 */
                switch (flag) {
                        case FLOW_ICMP_PACKET: /* ping  */
                                sin_src.sin_addr.s_addr = htonl(addr[(j++)/1000]);
                                //sin_src.sin_port = htons(port);
				sin_src.sin_port = htons(200);
                                break;
			case SCAN_ICMP_PACKET:
                                sin_src.sin_addr.s_addr = htonl(addr[1]);
                                sin_src.sin_port = htons(port);
                                //sin_dest.sin_addr.s_addr = htonl(addr[(j++)%COUNT]);
                                sin_dest.sin_port = htons(portno[(j++)%COUNT]);
                                break;
                }
                /* construct a icmp packet */
                icmp_packet(&sin_src, &sin_dest, &pkt, type);
	        if (setsockopt(sock, IPPROTO_IP, IP_HDRINCL, &hincl, sizeof(hincl)))
        	{
          		fprintf(stderr,"Unable to set IP_HDRINCL to 1: %s",strerror(errno));
          		exit(-1);
        	}

                if(sendto(sock, pkt.buffer, pkt.pkt_len, 0,
                  (struct sockaddr *) &sin_ddest, sizeof(struct sockaddr_in))<0)
                {
                        fprintf(stderr,"couldn't send packet:%s\n",
                                strerror(errno));
                        exit(-1);
                }
         }

}



/* send syn packet to ip 
 */
void tworm(char *host, int port)
{
	fprintf(stderr,"tworm:");
	if(tworm_flag&TWORM_SYN) {
		fprintf(stderr," syn ");
	} 
	if(tworm_flag&TWORM_FIN) {
		fprintf(stderr," fin ");
	}
	fprintf(stderr, "\n");

	/* generate and send tons of tcp packets */
	tcp_horse(host, port, TWORM_PACKET);	
}


/* test topn_ip trigger */
void flow(char *host, int port)
{
	int i;

	fprintf(stderr,"flow: ");
	if(flow_flag & FLOW_TCP) {
		fprintf(stderr ," tcp\t");
		tcp_horse(host, port, FLOW_TCP_PACKET);
	}
	if(flow_flag & FLOW_UDP) {
		fprintf(stderr, " udp\t");
		udp_horse(host, port, FLOW_UDP_PACKET);
	}
	if(flow_flag & FLOW_IP) {
		fprintf(stderr, " ip");
		if(rand() % 2 == 0) {
			fprintf(stderr,"\ttcp");
			tcp_horse(host, port, FLOW_TCP_PACKET);
		}
		else if(rand() % 3 == 0) {
			fprintf(stderr,"\tudp");
			udp_horse(host, port, FLOW_UDP_PACKET);
		}
		else if(rand() % 5 == 0) {
			fprintf(stderr,"\ticmp");
			icmp_horse(host, port, ECHO, FLOW_ICMP_PACKET);
		}
	}
	fprintf(stderr, "\n");
}

void scan(char *host, int port)
{
	fprintf(stderr,"scan:");
	if(scan_flag&SCAN_IP) {
		udp_horse(host, port, SCAN_UDP_PACKET);
	}
	if(scan_flag&SCAN_PORT){
		udp_horse(host, port, SCAN_UPORT_PACKET);
	}
	if(scan_flag&SCAN_TPORT){
		tcp_horse(host, port, SCAN_TPORT_PACKET);
	}
	if(scan_flag&SCAN_UPORT){
		udp_horse(host, port, SCAN_UPORT_PACKET);
	}
}

void udperror(char *host, int port)
{
	udp_horse(host, port, SCAN_UPORT_PACKET);
	udp_horse(host, port, UDP_ERROR_PACKET);
	icmp_horse(host, port, UNREACHABLE, SCAN_ICMP_PACKET);
}

int main(int argc, char **argv)
{
	char ip[32], host[32];
	int port;

	if( argc == 1 ) {
		print_usage();
	}

	/* parse the parameters */
	parse_para(argv, argc, host, &port); 
 
	if(tworm_flag) {
		tworm(host, port);
	}
	if(flow_flag) {
		flow(host, port);
	}
	if(scan_flag) {
		scan(host, port);
	}
	if(udperror_flag) {
		udperror(host, port);
	}
	
	return 0;
}

